@@ -1,4 +1,4 @@
-/* $OpenBSD: dh_key.c,v 1.29 2018/06/12 15:33:18 sthen Exp $ */
+/* $OpenBSD: dh_key.c,v 1.36 2018/11/12 17:39:17 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -102,11 +102,10 @@ static int
 generate_key(DH *dh)
 {
 	int ok = 0;
-	int generate_new_key = 0;
 	unsigned l;
 	BN_CTX *ctx;
 	BN_MONT_CTX *mont = NULL;
-	BIGNUM *pub_key = NULL, *priv_key = NULL;
+	BIGNUM *pub_key = NULL, *priv_key = NULL, *two = NULL;
 
 	if (BN_num_bits(dh->p) > OPENSSL_DH_MAX_MODULUS_BITS) {
 		DHerror(DH_R_MODULUS_TOO_LARGE);
@@ -117,20 +116,15 @@ generate_key(DH *dh)
 	if (ctx == NULL)
 		goto err;
 
-	if (dh->priv_key == NULL) {
-		priv_key = BN_new();
-		if (priv_key == NULL)
+	if ((priv_key = dh->priv_key) == NULL) {
+		if ((priv_key = BN_new()) == NULL)
 			goto err;
-		generate_new_key = 1;
-	} else
-		priv_key = dh->priv_key;
+	}
 
-	if (dh->pub_key == NULL) {
-		pub_key = BN_new();
-		if (pub_key == NULL)
+	if ((pub_key = dh->pub_key) == NULL) {
+		if ((pub_key = BN_new()) == NULL)
 			goto err;
-	} else
-		pub_key = dh->pub_key;
+	}
 
 	if (dh->flags & DH_FLAG_CACHE_MONT_P) {
 		mont = BN_MONT_CTX_set_locked(&dh->method_mont_p,
@@ -139,12 +133,14 @@ generate_key(DH *dh)
 			goto err;
 	}
 
-	if (generate_new_key) {
+	if (dh->priv_key == NULL) {
 		if (dh->q) {
-			do {
-				if (!BN_rand_range(priv_key, dh->q))
-					goto err;
-			} while (BN_is_zero(priv_key) || BN_is_one(priv_key));
+			if ((two = BN_new()) == NULL)
+				goto err;
+			if (!BN_add(two, BN_value_one(), BN_value_one()))
+				goto err;
+			if (!bn_rand_interval(priv_key, two, dh->q))
+				goto err;
 		} else {
 			/* secret exponent length */
 			l = dh->length ? dh->length : BN_num_bits(dh->p) - 1;
@@ -153,30 +149,23 @@ generate_key(DH *dh)
 		}
 	}
 
-	{
-		BIGNUM prk;
-
-		BN_init(&prk);
-		BN_with_flags(&prk, priv_key, BN_FLG_CONSTTIME);
-
-		if (!dh->meth->bn_mod_exp(dh, pub_key, dh->g, &prk, dh->p, ctx,
-		    mont)) {
-			goto err;
-		}
-	}
+	if (!dh->meth->bn_mod_exp(dh, pub_key, dh->g, priv_key, dh->p, ctx,
+	    mont))
+		goto err;
 
 	dh->pub_key = pub_key;
 	dh->priv_key = priv_key;
 	ok = 1;
-err:
+ err:
 	if (ok != 1)
 		DHerror(ERR_R_BN_LIB);
 
-	if (pub_key != NULL && dh->pub_key == NULL)
+	if (dh->pub_key == NULL)
 		BN_free(pub_key);
-	if (priv_key != NULL && dh->priv_key == NULL)
+	if (dh->priv_key == NULL)
 		BN_free(priv_key);
 	BN_CTX_free(ctx);
+	BN_free(two);
 	return ok;
 }
 
@@ -228,7 +217,7 @@ compute_key(unsigned char *key, const BIGNUM *pub_key, DH *dh)
 	}
 
 	ret = BN_bn2bin(tmp, key);
-err:
+ err:
 	if (ctx != NULL) {
 		BN_CTX_end(ctx);
 		BN_CTX_free(ctx);
