@@ -1,4 +1,4 @@
-/* $OpenBSD: d1_pkt.c,v 1.85 2020/10/03 17:35:16 jsing Exp $ */
+/* $OpenBSD: d1_pkt.c,v 1.93 2021/02/20 14:14:16 tb Exp $ */
 /*
  * DTLS implementation written by Nagendra Modadugu
  * (nagendra@cs.stanford.edu) for the OpenSSL project 2005.
@@ -184,8 +184,10 @@ satsub64be(const unsigned char *v1, const unsigned char *v2)
 
 static int have_handshake_fragment(SSL *s, int type, unsigned char *buf,
     int len, int peek);
-static int dtls1_record_replay_check(SSL *s, DTLS1_BITMAP *bitmap);
-static void dtls1_record_bitmap_update(SSL *s, DTLS1_BITMAP *bitmap);
+static int dtls1_record_replay_check(SSL *s, DTLS1_BITMAP *bitmap,
+    const unsigned char *seq);
+static void dtls1_record_bitmap_update(SSL *s, DTLS1_BITMAP *bitmap,
+    const unsigned char *seq);
 static DTLS1_BITMAP *dtls1_get_bitmap(SSL *s, SSL3_RECORD_INTERNAL *rr,
     unsigned int *is_next_epoch);
 static int dtls1_buffer_record(SSL *s, record_pqueue *q,
@@ -203,9 +205,6 @@ dtls1_copy_record(SSL *s, DTLS1_RECORD_DATA_INTERNAL *rdata)
 	memcpy(&(S3I(s)->rbuf), &(rdata->rbuf), sizeof(SSL3_BUFFER_INTERNAL));
 	memcpy(&(S3I(s)->rrec), &(rdata->rrec), sizeof(SSL3_RECORD_INTERNAL));
 
-	/* Set proper sequence number for mac calculation */
-	memcpy(&(S3I(s)->read_sequence[2]), &(rdata->packet[5]), 6);
-
 	return (1);
 }
 
@@ -316,7 +315,6 @@ dtls1_process_record(SSL *s)
 	size_t out_len;
 
 	tls12_record_layer_set_version(s->internal->rl, s->version);
-	tls12_record_layer_set_read_epoch(s->internal->rl, rr->epoch);
 
 	if (!tls12_record_layer_open_record(s->internal->rl, s->internal->packet,
 	    s->internal->packet_length, &out, &out_len)) {
@@ -330,7 +328,7 @@ dtls1_process_record(SSL *s)
 		else if (alert_desc == SSL_AD_BAD_RECORD_MAC)
 			SSLerror(s, SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC);
 
-		goto f_err;
+		goto fatal_err;
 	}
 
 	rr->data = out;
@@ -341,7 +339,7 @@ dtls1_process_record(SSL *s)
 
 	return (1);
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, alert_desc);
  err:
 	return (0);
@@ -415,12 +413,13 @@ again:
 		    !CBS_get_bytes(&header, &seq_no, 6))
 			goto again;
 
-		if (!CBS_write_bytes(&seq_no, &(S3I(s)->read_sequence[2]),
-		    sizeof(S3I(s)->read_sequence) - 2, NULL))
-			goto again;
 		if (!CBS_get_u16(&header, &len))
 			goto again;
 
+		if (!CBS_write_bytes(&seq_no, &rr->seq_num[2],
+		    sizeof(rr->seq_num) - 2, NULL))
+			goto again;
+
 		rr->type = type;
 		rr->epoch = epoch;
 		rr->length = len;
@@ -467,7 +466,7 @@ again:
 	 */
 	if (!(D1I(s)->listen && rr->type == SSL3_RT_HANDSHAKE &&
 	    p != NULL && *p == SSL3_MT_CLIENT_HELLO) &&
-	    !dtls1_record_replay_check(s, bitmap))
+	    !dtls1_record_replay_check(s, bitmap, rr->seq_num))
 		goto again;
 
 	/* just read a 0 length packet */
@@ -485,7 +484,7 @@ again:
 			    rr->seq_num) < 0)
 				return (-1);
 			/* Mark receipt of record. */
-			dtls1_record_bitmap_update(s, bitmap);
+			dtls1_record_bitmap_update(s, bitmap, rr->seq_num);
 		}
 		goto again;
 	}
@@ -494,7 +493,7 @@ again:
 		goto again;
 
 	/* Mark receipt of record. */
-	dtls1_record_bitmap_update(s, bitmap);
+	dtls1_record_bitmap_update(s, bitmap, rr->seq_num);
 
 	return (1);
 }
@@ -576,16 +575,8 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 	 * so process data buffered during the last handshake
 	 * in advance, if any.
 	 */
-	if (S3I(s)->hs.state == SSL_ST_OK && rr->length == 0) {
-		pitem *item;
-		item = pqueue_pop(D1I(s)->buffered_app_data.q);
-		if (item) {
-			dtls1_copy_record(s, item->data);
-
-			free(item->data);
-			pitem_free(item);
-		}
-	}
+	if (S3I(s)->hs.state == SSL_ST_OK && rr->length == 0)
+		dtls1_retrieve_buffered_record(s, &(D1I(s)->buffered_app_data));
 
 	/* Check for timeout */
 	if (dtls1_handle_timeout(s) > 0)
@@ -636,16 +627,15 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		return (0);
 	}
 
-
-	if (type == rr->type) /* SSL3_RT_APPLICATION_DATA or SSL3_RT_HANDSHAKE */
-	{
+	/* SSL3_RT_APPLICATION_DATA or SSL3_RT_HANDSHAKE */
+	if (type == rr->type) {
 		/* make sure that we are not getting application data when we
 		 * are doing a handshake for the first time */
-		if (SSL_in_init(s) && (type == SSL3_RT_APPLICATION_DATA) &&
-			(s->enc_read_ctx == NULL)) {
+		if (SSL_in_init(s) && type == SSL3_RT_APPLICATION_DATA &&
+		    !tls12_record_layer_read_protected(s->internal->rl)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_APP_DATA_IN_HANDSHAKE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (len <= 0)
@@ -708,7 +698,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 			/* Not certain if this is the right error handling */
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		if (dest_maxlen > 0) {
@@ -745,7 +735,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		    (D1I(s)->handshake_fragment[3] != 0)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_BAD_HELLO_REQUEST);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		/* no need to check sequence number on HELLO REQUEST messages */
@@ -831,7 +821,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		} else {
 			al = SSL_AD_ILLEGAL_PARAMETER;
 			SSLerror(s, SSL_R_UNKNOWN_ALERT_TYPE);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		goto start;
@@ -857,7 +847,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		    (rr->off != 0) || (rr->data[0] != SSL3_MT_CCS)) {
 			al = SSL_AD_DECODE_ERROR;
 			SSLerror(s, SSL_R_BAD_CHANGE_CIPHER_SPEC);
-			goto f_err;
+			goto fatal_err;
 		}
 
 		rr->length = 0;
@@ -951,7 +941,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		}
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-		goto f_err;
+		goto fatal_err;
 	case SSL3_RT_CHANGE_CIPHER_SPEC:
 	case SSL3_RT_ALERT:
 	case SSL3_RT_HANDSHAKE:
@@ -960,7 +950,7 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		 * should not happen when type != rr->type */
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
-		goto f_err;
+		goto fatal_err;
 	case SSL3_RT_APPLICATION_DATA:
 		/* At this point, we were expecting handshake data,
 		 * but have application data.  If the library was
@@ -982,12 +972,12 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		} else {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 	/* not reached */
 
- f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
  err:
 	return (-1);
@@ -1100,7 +1090,6 @@ do_dtls1_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 		goto err;
 
 	tls12_record_layer_set_version(s->internal->rl, s->version);
-	tls12_record_layer_set_write_epoch(s->internal->rl, D1I(s)->w_epoch);
 
 	if (!tls12_record_layer_seal_record(s->internal->rl, type, buf, len, &cbb))
 		goto err;
@@ -1129,34 +1118,30 @@ do_dtls1_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 }
 
 static int
-dtls1_record_replay_check(SSL *s, DTLS1_BITMAP *bitmap)
+dtls1_record_replay_check(SSL *s, DTLS1_BITMAP *bitmap,
+    const unsigned char *seq)
 {
-	int cmp;
 	unsigned int shift;
-	const unsigned char *seq = S3I(s)->read_sequence;
+	int cmp;
 
 	cmp = satsub64be(seq, bitmap->max_seq_num);
-	if (cmp > 0) {
-		memcpy (S3I(s)->rrec.seq_num, seq, 8);
+	if (cmp > 0)
 		return 1; /* this record in new */
-	}
 	shift = -cmp;
 	if (shift >= sizeof(bitmap->map)*8)
 		return 0; /* stale, outside the window */
 	else if (bitmap->map & (1UL << shift))
 		return 0; /* record previously received */
 
-	memcpy(S3I(s)->rrec.seq_num, seq, 8);
 	return 1;
 }
 
-
 static void
-dtls1_record_bitmap_update(SSL *s, DTLS1_BITMAP *bitmap)
+dtls1_record_bitmap_update(SSL *s, DTLS1_BITMAP *bitmap,
+    const unsigned char *seq)
 {
-	int cmp;
 	unsigned int shift;
-	const unsigned char *seq = S3I(s)->read_sequence;
+	int cmp;
 
 	cmp = satsub64be(seq, bitmap->max_seq_num);
 	if (cmp > 0) {
@@ -1173,7 +1158,6 @@ dtls1_record_bitmap_update(SSL *s, DTLS1_BITMAP *bitmap)
 	}
 }
 
-
 int
 dtls1_dispatch_alert(SSL *s)
 {
@@ -1237,19 +1221,13 @@ dtls1_get_bitmap(SSL *s, SSL3_RECORD_INTERNAL *rr, unsigned int *is_next_epoch)
 void
 dtls1_reset_seq_numbers(SSL *s, int rw)
 {
-	unsigned char *seq;
-	unsigned int seq_bytes = sizeof(S3I(s)->read_sequence);
-
 	if (rw & SSL3_CC_READ) {
 		D1I(s)->r_epoch++;
-		seq = S3I(s)->read_sequence;
-		memcpy(&(D1I(s)->bitmap), &(D1I(s)->next_bitmap), sizeof(DTLS1_BITMAP));
+		memcpy(&(D1I(s)->bitmap), &(D1I(s)->next_bitmap),
+		    sizeof(DTLS1_BITMAP));
 		memset(&(D1I(s)->next_bitmap), 0, sizeof(DTLS1_BITMAP));
 	} else {
 		D1I(s)->w_epoch++;
-		seq = S3I(s)->write_sequence;
-		memcpy(D1I(s)->last_write_sequence, seq, sizeof(S3I(s)->write_sequence));
+		tls12_record_layer_set_write_epoch(s->internal->rl, D1I(s)->w_epoch);
 	}
-
-	memset(seq, 0, seq_bytes);
 }
